import maya.cmds as cmds
import maya.mel as mel
import math

widgets = {}

metalPresets = [('Default',0,0,0,0,0,0),
                ('Aluminium',1.55803,7.7124,0.84921,6.1648,0.72122,5.7556),
                ('Gold',0.17009,3.1421,0.7062,2.0307,1.26175,1.8014),
                ('Copper',0.21845,3.6370,1.12497,2.5834,1.15106,2.4926),
                ('Silver',0.13928,4.1285,0.13,3.0094,0.13329,2.7028),
                ('Chromium',3.1044,3.3274,2.85932,3.3221,2.54232,3.2521),
                ('Platinum',2.37387,4.2454,2.00768,3.5017,1.90571,3.2893),
                ('Nickel',2.01294,3.8059,1.69725,3.0186,1.65785,2.7993),
                ('Steel',2.1,3.4,2.25,3.5,2.8,3.82),
                ('Bronze',1.75,3.5,1.9,3.8,2.45,4.8),
                ('Brass',4,2.2,4.3,2.2,5.25,2.0)
               ]

def UI():

    if cmds.window('fresnel_controls', q=True, exists=True):
        cmds.deleteUI('fresnel_controls')
        cmds.windowPref('fresnel_controls', remove=True)  # kill prefs
    
    # make the window
    win = cmds.window('fresnel_controls',
                    title       = "Fresnel Tools",
                    sizeable    = True,
                    retain      = False,
                    toolbox     = True,
                    widthHeight = (315, 180))

    # MAIN LAYOUT
    widgets['mainLayout'] = cmds.columnLayout(adj=1)


    # Create form layout
    widgets['formLayout_01'] = cmds.formLayout()

    """
    +-----------------------------------------------------------+
    |------------------------IOR FRESNEL------------------------|
    +-----------------------------------------------------------+
    """

    reduceMargin = 350
    indent = 5

    widgets['FresnelImg'] = cmds.image( image='fresnel.png',w=300,h=43 )

    widgets['nText'] = cmds.text( label='N',fn='plainLabelFont')
    widgets['nVal'] = cmds.floatField( 'nVal',minValue=0, maxValue=200000, value=0, w=50, precision=5) #cc=?

    widgets['kText'] = cmds.text( label='K',fn='plainLabelFont')
    widgets['kVal'] = cmds.floatField( 'kVal',minValue=0, maxValue=200000, value=0, w=50, precision=5)

    widgets['iorBtn'] = cmds.button('Create',w=50,h=20,ebg=0,bgc= (0.1,0.27,0.47), command=drawCurve)

    widgets['sepIor'] = cmds.separator( w=315,height=2, style='out' )

    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['FresnelImg'],'top',360-reduceMargin), (widgets['FresnelImg'],'left',5) ])

    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['nText'],'top',413-reduceMargin), (widgets['nText'],'left',10+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['nVal'],'top',410-reduceMargin), (widgets['nVal'],'left',20+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['kText'],'top',413-reduceMargin), (widgets['kText'],'left',80+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['kVal'],'top',410-reduceMargin), (widgets['kVal'],'left',90+indent) ])

    
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['iorBtn'],'top',410-reduceMargin), (widgets['iorBtn'],'left',186) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['sepIor'],'top',435-reduceMargin), (widgets['sepIor'],'left',0) ])

    """
    +-----------------------------------------------------------+
    |--------------------IOR FRESNEL TRIPLE---------------------|
    +-----------------------------------------------------------+
    """
    widgets['iorRed'] = cmds.separator( w=2,height=18, style='none',bgc=(0.8,0,0) )
    widgets['nTextRED'] = cmds.text( label='N',fn='plainLabelFont')
    widgets['nValRED'] = cmds.floatField( 'nValRED',minValue=0, maxValue=200000, value=0, w=50, precision=5)
    widgets['kTextRED'] = cmds.text( label='K',fn='plainLabelFont')
    widgets['kValRED'] = cmds.floatField( 'kValRED',minValue=0, maxValue=200000, value=0, w=50, precision=5)

    widgets['iorGreen'] = cmds.separator( w=2,height=18, style='none',bgc=(0,0.8,0) )
    widgets['nTextGREEN'] = cmds.text( label='N',fn='plainLabelFont')
    widgets['nValGREEN'] = cmds.floatField( 'nValGREEN',minValue=0, maxValue=200000, value=0, w=50, precision=5)
    widgets['kTextGREEN'] = cmds.text( label='K',fn='plainLabelFont')
    widgets['kValGREEN'] = cmds.floatField( 'kValGREEN',minValue=0, maxValue=200000, value=0, w=50, precision=5)

    widgets['iorBlue'] = cmds.separator( w=2,height=18, style='none',bgc=(0,0,0.8) )
    widgets['nTextBLUE'] = cmds.text( label='N',fn='plainLabelFont')
    widgets['nValBLUE'] = cmds.floatField( 'nValBLUE',minValue=0, maxValue=200000, value=0, w=50, precision=5)
    widgets['kTextBLUE'] = cmds.text( label='K',fn='plainLabelFont')
    widgets['kValBLUE'] = cmds.floatField( 'kValBLUE',minValue=0, maxValue=200000, value=0, w=50, precision=5)

    widgets['opmenuIor'] = cmds.optionMenu( 'opmenuIor',w=75,cc=presetFresnel )
    for preset in metalPresets:
        itemsMenuIor = cmds.menuItem(label=preset[0] )

    widgets['iorTripleBtn'] = cmds.button('Create Network',w=100,h=24,ebg=0,bgc= (0.1,0.27,0.47), command=drawCurve2)

    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['nTextRED'],'top',448-reduceMargin), (widgets['nTextRED'],'left',10+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['nValRED'],'top',445-reduceMargin), (widgets['nValRED'],'left',20+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['kTextRED'],'top',448-reduceMargin), (widgets['kTextRED'],'left',80+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['kValRED'],'top',445-reduceMargin), (widgets['kValRED'],'left',90+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['iorRed'],'top',446-reduceMargin), (widgets['iorRed'],'left',1+indent) ])

    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['nTextGREEN'],'top',472-reduceMargin), (widgets['nTextGREEN'],'left',10+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['nValGREEN'],'top',469-reduceMargin), (widgets['nValGREEN'],'left',20+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['kTextGREEN'],'top',472-reduceMargin), (widgets['kTextGREEN'],'left',80+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['kValGREEN'],'top',469-reduceMargin), (widgets['kValGREEN'],'left',90+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['iorGreen'],'top',470-reduceMargin), (widgets['iorGreen'],'left',1+indent) ])

    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['nTextBLUE'],'top',495-reduceMargin), (widgets['nTextBLUE'],'left',10+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['nValBLUE'],'top',493-reduceMargin), (widgets['nValBLUE'],'left',20+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['kTextBLUE'],'top',495-reduceMargin), (widgets['kTextBLUE'],'left',80+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['kValBLUE'],'top',493-reduceMargin), (widgets['kValBLUE'],'left',90+indent) ])
    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['iorBlue'],'top',494-reduceMargin), (widgets['iorBlue'],'left',1+indent) ])

    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['opmenuIor'],'top',440-reduceMargin), (widgets['opmenuIor'],'left',186) ])

    cmds.formLayout(widgets['formLayout_01'],e=1,af=[ (widgets['iorTripleBtn'],'top',488-reduceMargin), (widgets['iorTripleBtn'],'left', 186) ])


    cmds.showWindow(win)


"""
---------------
Fresnel Formula
---------------
"""

def IOR(n,k):

    theta_deg = 0

    n = n
    k = k
    fresnel = []

    while theta_deg <= 90:
        theta = math.radians(theta_deg)
        a = math.sqrt((math.sqrt((n**2-k**2-(math.sin(theta))**2)**2 + ((4 * n**2) * k**2)) + (n**2 - k**2 - (math.sin(theta))**2))/2)
        b = math.sqrt((math.sqrt((n**2-k**2-(math.sin(theta))**2)**2 + ((4 * n**2) * k**2)) - (n**2 - k**2 - (math.sin(theta))**2))/2)

        Fs = (a**2+b**2-(2 * a * math.cos(theta))+(math.cos(theta))**2)/ \
             (a**2+b**2+(2 * a * math.cos(theta))+(math.cos(theta))**2)
        Fp = Fs * ((a**2+b**2-(2 * a * math.sin(theta) * math.tan(theta))+(math.sin(theta))**2*(math.tan(theta))**2)/ \
              (a**2+b**2+(2 * a * math.sin(theta) * math.tan(theta))+(math.sin(theta))**2*(math.tan(theta))**2))
        R = (Fs + Fp)/2

        fresnel.append(R)

        theta_deg += 1
    return fresnel



"""
-----------------------
Ramer Douglas Algorithm
-----------------------
"""



def _vec2d_dist(p1, p2):
    return (p1[0] - p2[0])**2 + (p1[1] - p2[1])**2


def _vec2d_sub(p1, p2):
    return (p1[0]-p2[0], p1[1]-p2[1])


def _vec2d_mult(p1, p2):
    return p1[0]*p2[0] + p1[1]*p2[1]


def ramerdouglas(line, dist):

    if len(line) < 3:
        return line

    (begin, end) = (line[0], line[-1]) if line[0] != line[-1] else (line[0], line[-2])

    distSq = []
    for curr in line[1:-1]:
        tmp = (
            _vec2d_dist(begin, curr) - _vec2d_mult(_vec2d_sub(end, begin), _vec2d_sub(curr, begin)) ** 2 / _vec2d_dist(begin, end))
        distSq.append(tmp)

    maxdist = max(distSq)
    if maxdist < dist ** 2:
        return [begin, end]

    pos = distSq.index(maxdist)
    return (ramerdouglas(line[:pos + 2], dist) + 
            ramerdouglas(line[pos + 1:], dist)[1:])


"""
---------------------------
Draw Fresnel Curve | Single
---------------------------
"""


def drawCurve(*args):

    nValue = cmds.floatField( 'nVal' , q=1,v=True)
    kValue = cmds.floatField( 'kVal' , q=1,v=True)

    if nValue > 0:

        # create remapValue node
        remapNode = cmds.shadingNode('remapValue',asUtility=1)

        # Calculate Fresnel Curve
        fresnelList = IOR( nValue, kValue )


        # Compensate for non-linear facingRatio
        linearValues = [ float(i)/90 for i in range(91) ]
        rawValues = [ math.sin(linearValues[i]*90*math.pi/180) for i in range(91) ]
        rawValues.reverse()


        # Reduce curve points
        myline = zip(rawValues, fresnelList)
        precisionVals = [0.00005,0.0001,0.0002,0.0003]
        simplified = []

        for i in precisionVals:
            if len(simplified) == 0 or len(simplified) > 50:
                simplified = ramerdouglas(myline, dist = i)



        # Remove default values
        cmds.removeMultiInstance(remapNode+'.value[0]', b=1)
        cmds.removeMultiInstance(remapNode+'.value[1]', b=1)


        # Draw curve on remapValue Editor
        for i in simplified:
            currentSize = cmds.getAttr(remapNode +'.value',size=1)

            # First and last values with Linear interpolation
            if simplified.index(i) == 0 or simplified.index(i) == len(simplified)-1:
                cmds.setAttr( remapNode+'.value['+str( currentSize+1 )+']', i[0],i[1],1, type="double3")
            # Others with Spline interpolation
            else:
                cmds.setAttr( remapNode+'.value['+str( currentSize+1 )+']', i[0],i[1],3, type="double3")

    else:
        cmds.warning('N value must be greater than 0!')








def drawCurve2(*args):

    if cmds.nodeType( cmds.ls(sl=1) ) != 'mia_material_x_passes' and cmds.nodeType( cmds.ls(sl=1) ) != 'mia_material_x':
        cmds.warning('Select a mia_material!')
        return False

    aiMetal = cmds.ls(sl=1)[0]

    redValues = [cmds.floatField( 'nValRED' , q=1,v=True),cmds.floatField( 'kValRED' , q=1,v=True)]
    greenValues = [cmds.floatField( 'nValGREEN' , q=1,v=True),cmds.floatField( 'kValGREEN' , q=1,v=True)]
    blueValues = [cmds.floatField( 'nValBLUE' , q=1,v=True),cmds.floatField( 'kValBLUE' , q=1,v=True)]

    allNnK = [redValues,greenValues,blueValues]

    if all(i > 0 for i in redValues) and all(i > 0 for i in greenValues) and all(i > 0 for i in blueValues):
    
        # create additional nodes
        sInfo   = cmds.shadingNode('samplerInfo',asUtility=1)
        remapNodes = ['RED','GREEN','BLUE']


        # Compensate for non-linear facingRatio
        linearValues = [ float(i)/90 for i in range(91) ]
        rawValues = [ math.sin(linearValues[i]*90*math.pi/180) for i in range(91) ]
        rawValues.reverse()
        
        finalRemapNodes = []
        
        for remap in remapNodes:
            remapNode = cmds.shadingNode('remapValue', asUtility=1,n='remap_'+str(remap)+'')     
            finalRemapNodes.append(remapNode)

            fresnelList = IOR( allNnK[remapNodes.index(remap)][0] , allNnK[remapNodes.index(remap)][1] )

            # Reduce curve points
            myline = zip(rawValues, fresnelList)
            precisionVals = [0.00005,0.0001,0.0002,0.0003]
            simplified = []

            for i in precisionVals:
                if len(simplified) == 0 or len(simplified) > 50:
                    simplified = ramerdouglas(myline, dist = i)

            # remove default values
            cmds.removeMultiInstance(remapNode+'.value[0]', b=1)
            cmds.removeMultiInstance(remapNode+'.value[1]', b=1)

            # Draw curve on remapValue Editor
            for i in simplified:
                currentSize = cmds.getAttr(remapNode +'.value',size=1)

                # First and last values with Linear interpolation
                if simplified.index(i) == 0 or simplified.index(i) == len(simplified)-1:
                    cmds.setAttr( remapNode+'.value['+str( currentSize+1 )+']', i[0],i[1],1, type="double3")
                # Others with Spline interpolation
                else:
                    cmds.setAttr( remapNode+'.value['+str( currentSize+1 )+']', i[0],i[1],3, type="double3")

            
            # connect to network
            cmds.connectAttr(sInfo+'.facingRatio', remapNode+'.inputValue',f=1)            
            

        # connect to material        
        cmds.connectAttr( finalRemapNodes[0]+'.outValue', aiMetal+'.refl_colorR', f=1 )
        cmds.connectAttr( finalRemapNodes[1]+'.outValue', aiMetal+'.refl_colorG', f=1 )
        cmds.connectAttr( finalRemapNodes[2]+'.outValue', aiMetal+'.refl_colorB', f=1 )
        
        # set proper defaults        
        #cmds.setAttr('%s.refl_color' %aiMetal, 1,1,1, type='double3')
        cmds.setAttr(aiMetal+'.reflectivity', 1)
        #cmds.setAttr(aiMetal+'.refl_is_metal', 1)
        cmds.setAttr(aiMetal+'.brdf_fresnel', 0)
        cmds.setAttr(aiMetal+'.brdf_0_degree_refl', 1)



    else:
        cmds.warning('N and K values must be greater than 0!')





def presetFresnel(*args):

    selected = cmds.optionMenu( 'opmenuIor',q=1, sl=1 )-1

    cmds.floatField( 'nValRED',e=1,v=metalPresets[selected][1] )
    cmds.floatField( 'kValRED',e=1,v=metalPresets[selected][2] )
    cmds.floatField( 'nValGREEN',e=1,v=metalPresets[selected][3] )
    cmds.floatField( 'kValGREEN',e=1,v=metalPresets[selected][4] )
    cmds.floatField( 'nValBLUE',e=1,v=metalPresets[selected][5] )
    cmds.floatField( 'kValBLUE',e=1,v=metalPresets[selected][6] )